from collections import namedtuple
from typing import Tuple, List, Union

from gcma.io.param import Param, TParams

ConstraintValue = namedtuple("ConstraintValue", ("name", "value"))


class Constraint:
    TConstrainedParams = None

    def __init__(self, name: str, **coefs):
        self.name = name
        self.coefs = coefs

    def extract_params(self, params: TParams) -> TConstrainedParams:
        raise NotImplementedError

    def get_value(self, params: TParams) -> float:
        raise NotImplementedError

    def get_constraint(self, params: TParams) -> ConstraintValue:
        return ConstraintValue(self.name, self.get_value(params))


TConstraints = Union[list, List[Constraint]]


# noinspection PyAbstractClass
class BiConstraint(Constraint):
    TConstrainedParams = Tuple[Param, Param]

    def __init__(self, name: str, p1: str, p2: str, **kwargs):
        super().__init__(name, **kwargs)
        self.p1 = p1
        self.p2 = p2

    def extract_params(self, params: TParams) -> TConstrainedParams:
        return params[self.p1].copy(), params[self.p2].copy()


class Constraints(list):
    def __init__(self, *constraints: Union[Constraint, TConstraints]):
        if len(constraints) > 0:
            if constraints[0] is None:
                constraints = []
            elif isinstance(constraints[0], Constraint):
                constraints = [constraints]
        super().__init__(*constraints)

    def apply(self, params: TParams) -> List[ConstraintValue]:
        return [x.get_constraint(params) for x in self]
