import logging
from typing import List, Union, Tuple

import numpy as np

from gcma.io.baseio import BaseIO, TValue, Bases
from gcma.io.utils import r


class Feature(BaseIO):
    def __init__(
        self, *args, ref: TValue, error: TValue = None, weight: TValue = 1, **kwargs
    ):
        super().__init__(*args, **kwargs)
        self.ref = ref
        self.weight = weight
        self._error = error

        self.registered.add("ref")
        self.registered.add("error")
        self.registered.add("weight")

    @property
    def error(self):
        if self._error is None:
            if self.value is not None:
                return self.error2
        else:
            return self._error

    @error.setter
    def error(self, value: TValue):
        self._error = value

    @property
    def error2(self) -> TValue:
        """
        Return (x - ref) ** 2
        """
        return np.square(self.value - self.ref)

    @property
    def error_rel(self) -> TValue:
        """
        Return relative error (x - ref) / ref
        If ref=0, returns self.error2
        """
        if self.ref == 0:
            return self.error2
        return (self.value - self.ref) / self.ref

    @property
    def table(self):
        return Features(self).table


class Features(Bases):
    inner_class = Feature

    def build_table(self) -> Tuple[Bases.THeader, Bases.TData]:
        header = ["Name", "Value", "Ref", "Error", "Weight"]
        data = [
            [x.name, r(x.value), r(x.ref), r(x.error), x.weight] for x in self.values()
        ]
        return header, data


TFeatures = Union[List[Feature], Features]

log = logging.getLogger(__name__)
