import logging
from typing import Tuple, List, Union

from gcma.io.baseio import BaseIO, TValue, Bases
from gcma.io.utils import r

_Bounds = Tuple[TValue, TValue]


class Param(BaseIO):
    def __init__(
        self, *args, initial_value: TValue = None, bounds: _Bounds = None, **kwargs
    ):
        super().__init__(*args, **kwargs)
        self.initial_value = initial_value
        self.bounds = bounds

        self.registered.add("initial_value")
        self.registered.add("bounds")

    @property
    def table(self):
        return Params(self).table


class Params(Bases):
    inner_class = Param

    def build_table(self) -> Tuple[Bases.THeader, Bases.TData]:
        header = ["Name", "Baseline", "Value", "Bounds"]
        data = [[x.name, x.initial_value, r(x.value), x.bounds] for x in self.values()]
        return header, data


TParams = Union[List[Param], Params]

log = logging.getLogger(__name__)
