from datetime import datetime
from typing import List, Union

import numpy as np
from dateutil import parser


class TermOptions(dict):
    _m_tol = "You must give tolerance values (Union[List[float], float]): `tols=...`"
    _m_date = "You must give a termination date (str): `max_date='...'`"
    _m_gd = "Date must be greater than today: '{}'"
    _m_tv = "Tolerance must be the same lenght as values: {} != {}"

    def __init__(
        self,
        max_date: str = None,
        tols: Union[List[float], float] = None,
        term_on_date: bool = None,
        term_on_tol: bool = None,
    ):
        if term_on_date is None:
            term_on_date = max_date is not None

        if term_on_tol is None:
            term_on_tol = tols is not None

        if term_on_date:
            assert max_date is not None, self._m_date
            md = parser.parse(max_date, dayfirst=True)
            assert md > datetime.now(), self._m_gd.format(md)
            max_date = str(md)

        if term_on_tol:
            assert tols is not None, self._m_tol

        super().__init__(
            term_on_date=term_on_date,
            term_on_tol=term_on_tol,
            max_date=max_date,
            tols=tols,
        )

    @classmethod
    def from_dict(cls, d):
        return cls(**d)

    def is_over(self) -> bool:
        if self["term_on_date"]:
            md = parser.parse(self["max_date"], dayfirst=True)
            return md < datetime.now()
        else:
            return False

    def is_tol(self, values: Union[np.ndarray, List[float]]) -> bool:
        if not self["term_on_tol"]:
            return False

        values = np.array(values)
        tols = np.squeeze(self["tols"])
        if tols.ndim == 0:
            tols = np.repeat(tols, len(values))
        else:
            assert len(tols) == len(values), self._m_tv.format(len(tols), len(values))

        for x, t in zip(values, tols):
            if x > t:
                return False
        return True
