import logging

import numpy as np

from gcma.io.baseio import TValue
from gcma.io.param import TParams


class Prior:
    def __init__(self, mean: TValue = None, cov: np.ndarray = None, gamma: float = 1):
        """
        :param gamma: weigth of the regularisation term
        """
        self.mean = None if mean is None else np.array(mean)
        self.cov = None if cov is None else np.array(cov)
        self.gamma = gamma

    def get_distance(self, params: TParams) -> float:
        """
        Get the distance between current parameters and prior knowledge

        :param params: non normalised params of one sample
        """
        if self.mean is None or self.cov is None:
            return 0
        no = (params.vals - self.mean)[:, np.newaxis]
        return (no.T @ np.linalg.inv(self.cov) @ no)[0, 0] * self.gamma

    def update(self, results: np.ndarray):
        """
        Updates prior based on database parameters distribution

        :param results: parameters value for each member of the database, size (n_patients, n_params)
        """
        self.mean = results.mean(axis=0)
        self.cov = np.cov(results.T)


log = logging.getLogger(__name__)
